/* Copyright (c) 2007, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
* Author: Hejun Wu (whjnn@cse.ust.hk)
* Date last modified:  08/02/07
*/      


#ifndef _EVENT_DETECTION_H 
#define _EVENT_DETECTION_H 

enum {QUERY_FIELD_SIZE = 8,
      COMMAND_SIZE = 8,
      STRING_SIZE = 8};

enum {
  kMAX_RESULTS = 4,
  AGG_DATA_LEN=28  //WARNING -- this has been picked to fit within the current
                   // message size ...
};




enum {
  kONE_SHOT = 0x7FFF, //instead of epoch duration, just drain all the results in a buffer
  NULL_QUERY_FIELD = 0x80,
  GROUP_FIELD = 0x7F, //magic code that indicates the value of this
                      //attribute is the aggregate group of a nested query
  TYPED_FIELD = 0x7E, //magic code that indicates the value of this
                      //attribute is just a typed field from a table
};



enum {
  MAX_FIELDS = 16,
  MAX_EXPRS = 16,
  kNO_QUERY = 0xFF

};


enum {
  EQ = 0,
  NEQ = 1,
  GT = 2,
  GE = 3,
  LT = 4,
  LE = 5
};


//field operators, for use in exprs
enum {
  FOP_NOOP = 0,
  FOP_TIMES = 1,
  FOP_DIVIDE = 2,
  FOP_ADD = 3,
  FOP_SUBTRACT = 4,
  FOP_MOD = 5,
  FOP_RSHIFT = 6
};


enum {
  kNO_GROUPING_FIELD = 0xFFFF
};



enum {
  kSEL = 0,
  kAGG = 1,
  kTEMP_AGG = 2,
};




enum {
  kFIRST_RESULT = 0xFF,
  
};

//enums for the QueryResult qrType (what kind of query result)
enum {
  kUNDEFINED = 0,
  kIS_AGG = 1,
  kNOT_AGG = 2,
  kAGG_SINGLE_FIELD = 3
};

  //in query message -- is this a field or an expression
  enum {
    kFIELD = 0,
    kEXPR =1,
    kBUF_MSG = 2,
    kEVENT_MSG = 3,
    kN_EPOCHS_MSG = 4,
    kDROP_TABLE_MSG = 5
  };

  // type of query message
  enum {ADD_MSG = 0,
	DEL_MSG = 1,
	MODIFY_MSG = 2,
	RATE_MSG = 3,
	DROP_TABLE_MSG = 5
  };

enum {
  CRC_FAILURE = 1, //source of loss for contention monitoring
  ACK_FAILURE = 2,
  SEND_FAILURE = 3,
  ENQUEUE_FAILURE = 4,
  SEND_BUSY_FAILURE = 5
};

//enums for adjust the sample rate based on observed contention
enum {
  HIGH_CONTENTION_THRESH = 20,
  LOW_CONTENTION_THRESH = 10
};



  /* ----------------------- Status Message ------------------------ */
enum { kMAX_QUERIES = 8 };


enum {
  kDATA_MESSAGE_ID = 240,	// XXX must be one of the amids in lib/Route/MultiHop.h
  kQUERY_MESSAGE_ID = 101, 
  kCOMMAND_MESSAGE_ID = 103,
  kQUERY_REQUEST_MESSAGE_ID = 104,
  kEVENT_MESSAGE_ID = 105,
  kSTATUS_MESSAGE_ID = 106,
  kDIRECTED_DATA_MESSAGE_ID = 107,
  kTEST_DATA_MESSAGE_ID = 108,
  kMSG_LEN = DATA_LENGTH,
  AM_QUERYRESULT = kDATA_MESSAGE_ID,
  AM_QUERYMESSAGE = kQUERY_MESSAGE_ID,
  AM_STATUSMESSAGE = kSTATUS_MESSAGE_ID,
  AM_UARTMSG = 1,
  AM_RTCMSG = 107,
  AM_TIME_SYNC_MSG = 207,
  MSG_TYPE_RREQ  = 208,
  MSG_TYPE_RREF  = 209,
  MSG_TYPE_RREC =  210, 

};


#define SYNC_DATA_OFFSET 4


#define isRoot() (TOS_LOCAL_ADDRESS == 0)

#define kSHORT_MSG_LEN  8
#ifdef PLATFORM_PC
#else
#define MAX_BUFFER_LEN  960
#endif

//the following are used as default intervals
#define ACTIVE_INTERVAL   1024
#define SLEEP_INTERVAL   1024
#define DEFAULT_TRANS_CYCLE   2000 //20000L //10s	//2048
#define SENDING_BIT  0x0040
#define kMS_PER_CLOCK_EVENT 64

//buffer lens
#define kSHORT_MSG_LEN  8
#define MAX_BUFFER_LEN  960
#define TEST_DATA_LEN (kSHORT_MSG_LEN+offsetof(TOS_MHopMsg, data))

//sensor types
#define MAX_SENSOR_TYPES 10

#define TEMP_SENSOR  0
#define LIGHT_SENSOR 2
#define MIC_SENSOR 6
#define ACCEL_X_SENSOR 3
#define ACCEL_Y_SENSOR 4
#define MAGIC_SENSOR 5
#define INVALID_SENSOR 0xff

#define JUMP_SPIKE_PATTERN 1
#define HORIZON_PATTERN 2
#define SLOPE_PATTERN 4
#define OSCILLATION_PATTERN 8

#define ALL_PATTERN 0
#define PATTERN_TYPES 8

#endif
