/* Copyright (c) 2007, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
* Author: Hejun Wu (whjnn@cse.ust.hk)
* Date last modified:  08/01/07
*/                                         

#ifndef __COMPILE_DEFINES__
#define __COMPILE_DEFINES__

//defines indicating which features are in use
#undef kUSE_MAGNETOMETER //include magnetometer attribute? 1596 bytes code, 55 bytes ram
#define kQUERY_SHARING //allow query sharing
#undef kFANCY_AGGS //use fancy aggregates
#undef kEEPROM_ATTR //enable the EEPROM attribute -- uses about 3 kb of code
#undef kCONTENT_ATTR //enable the contention attribute
#undef kRAW_MIC_ATTRS // enable raw microphone or tone detector attributes
#undef kLIFE_CMD
#undef kSUPPORTS_EVENTS //about 3k of code, 100 bytes of ram for event based queries
#define kSTATUS //200 bytes of code -- allow lists of running queries to be fetched over the UART
//whj for removal of led module
#define LED_ENABLE
#define TELOSB_VERSION
#ifdef TELOSB_VERSION
#undef MICA_SERIES //whj compact for telosb
#else 
#define MICA_SERIES
#endif

#ifndef MICA_SERIES  //whj compact for telosb
#undef kSTATUS //200 bytes of code -- allow lists of running queries to be fetched over the UART
#undef kQUERY_SHARING //disallow query sharing
#undef kMATCHBOX // eeprom 
#undef TABLE_SUPPORT //whj compact for telosb
#undef WDT_ENABLE //whj compact for telosb
#undef NETWORK_MONITOR_SUPPORT //whj compact for telosb
#undef ADDITIONAL_ATTR //whj compact for telosb
#undef NETWORK_ATTR //whj compact for telosb
#undef TINYDB_COMMAND_SUPPORT //whj compact for telosb
#undef LOCAL_DATA_AFFECT //whj compact for telosb
#undef AGGREGATE_SUPPORT //whj compact for telosb
#else
#define TABLE_SUPPORT
#define WDT_ENABLE
#define NETWORK_MONITOR_SUPPORT
#define ADDITIONAL_ATTR
#define NETWORK_ATTR
#define TINYDB_COMMAND_SUPPORT
#define LOCAL_DATA_AFFECT
#define AGGREGATE_SUPPORT
#endif

#if !defined(PLATFORM_PC)
#undef kMATCHBOX //enabled logging to EEPROM, 20k code, 489 bytes RAM
#undef kUART_DEBUGGER //allow output to a UART debugger
#endif

#define kUART_DEBUGGER //allow output to a UART debugger

/*            RAM     CODE
   MAG        55      1596
   SHARING
   AGGS
   EEPROM              ~3k
   UART       68       1684
   EVENTS     100      ~3k
   STATUS              ~200
 */

//Probably don't want to mess with the options below -- 
// they control parameters that are set byt he options above

#ifdef kMATCHBOX
//matchbox needs an extra buffer
# define NUMBUFFERS 3 //how many buffers does BufferC need / have?
//needs extra RAM, so allocate less to the multihop queue
# define MHOP_QUEUE_SIZE 4 
#else
# define NUMBUFFERS 2 //how many buffers does BufferC need / have?
# define MHOP_QUEUE_SIZE 8
#endif

#define SEND_QUEUE_SIZE (MHOP_QUEUE_SIZE + 2)

#define NETWORK_MULTIHOP 0
#define NETWORK_HSN		 1

//define HSN_ROUTING for HSN network
//#define HSN_ROUTING

#ifdef HSN_ROUTING
#define NETWORK_MODULE	TinyDBShim
#define NETWORK_MODULE_ID NETWORK_HSN
#endif

#ifndef NETWORK_MODULE
# define NETWORK_MODULE_ID NETWORK_MULTIHOP
# define NETWORK_MODULE NetworkMultiHop 
#endif

#if NETWORK_MODULE_ID==NETWORK_MULTIHOP
#  define HAS_ROUTECONTROL
#endif

#ifdef GENERICCOMM
# undef GENERICCOMM
#endif

#define GENERICCOMM GenericComm //whj Promiscuous


#define MAX_NUM_SERVICES 2 
#define LogicalTime SimpleTime
#define MH6_ROUTING

#ifdef kUART_DEBUGGER
#define DEBUG(b,l)  (call UartDebugger.writeBuffer((b),(l+4)))
#define DEBUG_BUF(bufPtr, len)  atomic{uint8_t idbg; if(!gbStatus) {gmDbgMsg[4] = len; \
				for(idbg = 0; idbg < len; idbg++) { \
					gmDbgMsg[5+idbg] = ((uint8_t*)bufPtr)[idbg]; \
				} \
					DEBUG(gmDbgMsg,len+1); \
				gbStatus = TRUE; \
				}\
		}

#else
#define DEBUG(b,l)
#define DEBUG_BUF(bufPtr, len) 
#endif

#ifdef LED_ENABLE
#define GREEN_LED_TOGGLE()  (call Leds.greenToggle())
#define RED_LED_TOGGLE()  (call Leds.redToggle())
#define RED_LED_ON() (call Leds.redOn())
#define YELLOW_LED_ON()  (call Leds.yellowOn())
#define YELLOW_LED_TOGGLE()  (call Leds.yellowToggle())
#define TURN_OFF_LEDS() (call Leds.set(0))
#else
#define GREEN_LED_TOGGLE()  
#define RED_LED_TOGGLE()  
#define RED_LED_ON() 
#define YELLOW_LED_TOGGLE() 
#define TURN_OFF_LEDS() 
#endif

#define SYNC_DATA_LEN  2

//#define ALL_IN_ONE
#define COLUMN_NUM 6
#define INTEL_COLUMN_NUM 2
#define ID_COLUMN 3
#define DATA_COLUMN 5 
#define NSEN_COLUMN 2
#define NSEN_TS_COLUMN 1

#define MAX_CACHE_SIZE 300
#define MAX_SIM_SIZE 800

#ifdef ALL_IN_ONE
	#define SENSOR_DATA_FNAME "sensor"
#else
#define SENSOR_DATA_FNAME "node"
#endif
#define MAX_TIME_RECORDS 8
#define MAX_NUM_CHD 16
#define MAX_TIME_OUT 800
#define OUTPUT_TIME
#define SLOPE_SEQ_SIZE 80
//#define THRESHOLD_MATCHING
#define THRESHOLD_OPR "<" 
#define THRESHOLD_CONST 479 
#if 0
enum {
 kROUTING_SERVICE_ID = 222,
  AM_MULTIHOPMSG = 250,
  // WHJ 050805 for CNP negotiation
  AM_SCHEDULE_REQ_MSG = 249,
  AM_SCHEDULE_CMD_MSG = 248,
  AM_SCHEDULE_INF_MSG = 247,  
  AM_SCHEDULE_BK_REQ_MSG = 246,
  AM_SCHEDULE_BK_CMD_MSG = 245,
  AM_SCHEDULE_BK_INF_MSG = 243,  

}; 
#endif
#define ONLINE_SENSING 
#endif
